/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { VorgangFormularButtonsE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-formular-buttons.e2e.components';
import { dropCollections } from 'apps/alfa-e2e/src/support/cypress-helper';
import { loginAsEmil } from 'apps/alfa-e2e/src/support/user-util';
import { buildVorgang, createVorgang, initVorgaenge, objectIds } from 'apps/alfa-e2e/src/support/vorgang-util';
import { SnackBarE2EComponent } from '../../../components/ui/snackbar.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, VorgangMessagesE2E, VorgangStatusE2E, vorgangStatusLabelE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { contains, exist, haveText, notExist } from '../../../support/cypress.util';

describe('Vorgang wiedereroeffnen', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
	const snackbar: SnackBarE2EComponent = mainPage.getSnackBar();

	const vorgangPage: VorgangPage = new VorgangPage();
	const vorgangFormularButtons: VorgangFormularButtonsE2EComponent = vorgangPage.getFormularButtons();

	const vorgangAbgeschlossenWiedereroeffnen: VorgangE2E = { ...createVorgang(), name: 'DoWiedereroeffnenOnAbgeschlossen', status: VorgangStatusE2E.ABGESCHLOSSEN };
	const vorgangAbgeschlossenWiedereroeffnenRevoke: VorgangE2E = { ...buildVorgang(objectIds[0], 'DoRevokeWiedereroeffnenOnAbgeschlossen'), status: VorgangStatusE2E.ABGESCHLOSSEN };

	before(() => {
		initVorgaenge([vorgangAbgeschlossenWiedereroeffnen, vorgangAbgeschlossenWiedereroeffnenRevoke]);

		loginAsEmil();

		waitForSpinnerToDisappear();
		exist(vorgangList.getRoot());
	})

	after(() => {
		dropCollections();
	})

	describe('vorgang in status abgeschlossen', () => {

		describe('wiedereroeffnen', () => {

			it('Open Vorgang-Detail-Page', () => {
				vorgangList.getListItem(vorgangAbgeschlossenWiedereroeffnen.name).getRoot().click();
				waitForSpinnerToDisappear();

				exist(vorgangPage.getVorgangDetailHeader().getRoot());
			})

			describe('by button', () => {

				it('should have status Abgeschlossen', () => {
					haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangAbgeschlossenWiedereroeffnen.status]);
				})

				it('should show snackBar', () => {
					vorgangFormularButtons.getWiedereroeffnenButton().click();
					waitForSpinnerToDisappear();

					exist(snackbar.getCloseButton());
					contains(snackbar.getMessage(), VorgangMessagesE2E.WIEDEREROEFFNET);
				})

				it('should close snackBar on close', () => {
					snackbar.getCloseButton().click();

					notExist(snackbar.getMessage());
				})

				it('should have status In Bearbeitung', () => {
					vorgangPage.getVorgangDetailHeader().getStatus().should('exist').should('have.text', vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG]);
				})

				it('back to vorgang list', () => {
					vorgangPage.getSubnavigation().getBackButton().click();
					waitForSpinnerToDisappear();

					exist(vorgangList.getRoot());
				})

				it('should have status In Bearbeitung', () => {
					haveText(vorgangList.getListItem(vorgangAbgeschlossenWiedereroeffnen.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG]);
				})
			})
		})

		describe('wiedereroeffnen und rückgängig machen', () => {

			it('Open Vorgang-Detail-Page', () => {
				vorgangList.getListItem(vorgangAbgeschlossenWiedereroeffnenRevoke.name).getRoot().click();
				waitForSpinnerToDisappear();

				exist(vorgangPage.getVorgangDetailHeader().getRoot());
			})

			describe('by icon-button', () => {

				it('should have status Abgeschlossen', () => {
					haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangAbgeschlossenWiedereroeffnenRevoke.status]);
				})

				it('should show snackBar', () => {
					vorgangPage.getSubnavigation().getWiedereroeffnenIconButton().click();
					waitForSpinnerToDisappear();

					exist(snackbar.getCloseButton());
					contains(snackbar.getMessage(), VorgangMessagesE2E.WIEDEREROEFFNET);
				})

				it('should close snackbar on revoke', () => {
					snackbar.getRevokeButton().click();
					waitForSpinnerToDisappear();

					notExist(snackbar.getMessage());
				})

				it('should show status Abgeschlossen after progressbar dissapear', () => {
					notExist(vorgangPage.getProgressBar());

					haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.ABGESCHLOSSEN]);
				})

				it('back to vorgang list', () => {
					vorgangPage.getSubnavigation().getBackButton().click();
					waitForSpinnerToDisappear();

					exist(vorgangList.getRoot());
				})

				it('should have status Abgeschlossen', () => {
					haveText(vorgangList.getListItem(vorgangAbgeschlossenWiedereroeffnenRevoke.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.ABGESCHLOSSEN]);
				})
			})
		})
	})
});
