/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { HeaderE2EComponent } from 'apps/alfa-e2e/src/page-objects/header.po';
import { VorgangListItemE2EComponent } from '../../../components/vorgang/vorgang-item.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { EingangE2E, EingangHeaderE2E, VorgangE2E, VorgangStatusE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections, dropSearchIndex } from '../../../support/cypress-helper';
import { enterWith, exist, notExist } from '../../../support/cypress.util';
import { UserRoleE2E, loginAsEmil } from '../../../support/user-util';
import { buildVorgang, createVorgang, initSearchIndex, initVorgaenge, objectIds } from '../../../support/vorgang-util';

describe.skip('VorgangList Suche for EA User', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
	const header: HeaderE2EComponent = mainPage.getHeader();

	const vorgangPage: VorgangPage = new VorgangPage();

	const requestId: string = '20201118365670866102';
	const eingangHeader: EingangHeaderE2E = { ...createVorgang().eingangs[0].header };
	const eingang: EingangE2E = { ...createVorgang().eingangs[0], header: { ...eingangHeader, requestId } }
	const vorgangToStay: VorgangE2E = { ...createVorgang(), name: 'VorgangToStay', eingangs: [eingang] };

	const vorgangToDisappear: VorgangE2E = { ...buildVorgang(objectIds[1], 'VorgangToDisappear'), status: VorgangStatusE2E.NEU };

	describe(`on user with role ${UserRoleE2E.EINHEITLICHER_ANSPRECHPARTNER}`, () => {

		const vorgangStayInList: VorgangListItemE2EComponent = vorgangList.getListItem(vorgangToStay.name);
		const vorgangDisappearInList: VorgangListItemE2EComponent = vorgangList.getListItem(vorgangToDisappear.name);

		before(() => {
			initVorgaenge([vorgangToStay, vorgangToDisappear])

			initSearchIndex([vorgangToStay, vorgangToDisappear])

			loginAsEmil();

			waitForSpinnerToDisappear();
			exist(vorgangList.getRoot());
		})

		after(() => {
			dropCollections();
			dropSearchIndex();
		})

		it('should filter list', () => {
			doSearch();
			waitForSpinnerToDisappear();

			exist(vorgangStayInList.getRoot());
			notExist(vorgangDisappearInList.getRoot());
		})

		it('should show filtered list after navigate to vorgang detail and back', () => {
			doSearch();
			waitForSpinnerToDisappear();

			exist(vorgangList.getRoot());
			vorgangStayInList.getRoot().click();
			exist(vorgangPage.getVorgangDetailHeader().getRoot());
			vorgangPage.getSubnavigation().getBackButton().click();
			waitForSpinnerToDisappear();

			exist(vorgangList.getRoot());
			exist(vorgangStayInList.getRoot());
			notExist(vorgangDisappearInList.getRoot());
		})

		function doSearch(): void {
			enterWith(header.getVorgangSearch().getInput(), vorgangToStay.name);
		}
	})
});