import {defineConfig} from "@pandacss/dev";

export default defineConfig({
  // Whether to use css reset
  preflight: true,

  // Where to look for your css declarations
  include: ["libs/ui-alfa/src/**/*.{js,jsx,ts,tsx}", "apps/showcase/src/**/*.{js,jsx,ts,tsx}"],

  // Files to exclude
  exclude: [],

  // define when which theme will be applied
  // see in semanticTokens for definitions of color
  conditions: {
    default: '[data-theme=default] &',
    light: '[data-mode-color=light] &',
    dark: '[data-mode-color=dark] &',
  },
  
  // Useful for theme customization
  theme: {
    extend: {},
    // customized themes
    semanticTokens: {
      colors: {
        $text: {
          value: {
            _default: { _light: "#000", _dark: '#fff' },
          }
        },
        $backgroundColor: {
          value: {
            _default: { _light: "#fff", _dark: "#000" },
          }
        },
        $hover: {
          value: {
            _default: { _light: "{colors.gray.200}", _dark: "{colors.gray.800}" }
          }
        }
      }
    }
  },

  // use costumized theme on body
  globalCss: {
    body: {
      color: '$text',
      backgroundColor: '$backgroundColor'
    }
  },

  // The output directory for your css system
  outdir: "styled-system",
});
