/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Component, Input } from '@angular/core';
import { BinaryFileResource, BinaryFileService } from '@alfa-client/binary-file-shared';
import { EMPTY_ARRAY, StateResource, createEmptyStateResource, doOnValidStateResource, isNotNil } from '@alfa-client/tech-shared';
import { Resource, getUrl } from '@ngxp/rest';
import { Observable, of } from 'rxjs';
import { tap } from 'rxjs/operators';

@Component({
	selector: 'alfa-binary-file-attachment-container',
	templateUrl: './binary-file-attachment-container.component.html',
	styleUrls: ['./binary-file-attachment-container.component.scss'],
})
export class BinaryFileAttachmentContainerComponent {

	@Input() formArrayName: string;
	@Input() uploadStateResource: StateResource<Resource>;
	@Input() linkRelUploadAttachment: string;
	@Input() set existFiles(value: BinaryFileResource[]) {
		this.fileList = isNotNil(value) ? value : EMPTY_ARRAY;
	}

	uploadInProgress$: Observable<StateResource<Resource>> = of(createEmptyStateResource<Resource>());
	fileList: BinaryFileResource[] = EMPTY_ARRAY

	constructor(private binaryFileService: BinaryFileService) { }

	public getFileUriList(): string[] {
		return this.fileList.map(file => getUrl(file));
	}

	public uploadFile(file: File): void {
		this.uploadInProgress$ = this.uploadAndGetFile(file);
	}

	uploadAndGetFile(file: File): Observable<StateResource<BinaryFileResource>> {
		return this.binaryFileService.uploadFile(this.uploadStateResource.resource, this.linkRelUploadAttachment, file).pipe(
			tap((stateResource: StateResource<BinaryFileResource>) => this.doAfterFileUpload(stateResource))
		);
	}

	doAfterFileUpload(stateResource: StateResource<BinaryFileResource>): void {
		doOnValidStateResource(stateResource, () => this.fileList = [...this.fileList, stateResource.resource]);
	}

	public deleteAttachment(binaryFile: BinaryFileResource): void {
		this.fileList = this.fileList.filter(file => getUrl(file) !== getUrl(binaryFile));
	}
}

