/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormatDateWithTimePipe, HasLinkPipe } from '@alfa-client/tech-shared';
import { getElementFromFixture } from '@alfa-client/test-utils';
import { AppIconComponent } from '@alfa-client/ui';
import { UserProfileInHistorieContainerComponent } from '@alfa-client/user-profile';
import { CommandLinkRel } from 'libs/command-shared/src/lib/command.linkrel';
import { createCommandResource } from 'libs/command-shared/test/command';
import { getDataTestClassOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { HistorieItemHeaderComponent } from './historie-item-header.component';

registerLocaleData(localeDe);

describe('HistorieItemHeaderComponent', () => {
	let component: HistorieItemHeaderComponent;
	let fixture: ComponentFixture<HistorieItemHeaderComponent>;

	const userProfile: string = getDataTestClassOf('user-profile-in-historie-item-header');
	const nonUserCommandText: string = getDataTestClassOf('system-user-in-historie-item-header');


	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				FormatDateWithTimePipe,
				HasLinkPipe,
				HistorieItemHeaderComponent,
				MockComponent(UserProfileInHistorieContainerComponent),
				MockComponent(AppIconComponent)
			],
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(HistorieItemHeaderComponent);
		component = fixture.componentInstance;
		component.resource = createCommandResource();
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	})

	describe('user profile', () => {

		it('should show if link is present', () => {
			component.resource = createCommandResource([CommandLinkRel.CREATED_BY]);
			fixture.detectChanges();

			const element = getElementFromFixture(fixture, userProfile);

			expect(element).toBeInstanceOf(HTMLElement);
		})

		it('should NOT show if link not exists', () => {
			component.resource = createCommandResource();
			fixture.detectChanges();

			const element = getElementFromFixture(fixture, userProfile);

			expect(element).not.toBeInstanceOf(HTMLElement);
		})
	})

	describe('non user text', () => {

		it('should show if link NOT exists', () => {
			component.resource = createCommandResource();
			fixture.detectChanges();

			const element = getElementFromFixture(fixture, nonUserCommandText);

			expect(element).toBeInstanceOf(HTMLElement);
		})

		it('should NOT show if link is present', () => {
			component.resource = createCommandResource([CommandLinkRel.CREATED_BY]);
			fixture.detectChanges();

			const element = getElementFromFixture(fixture, nonUserCommandText);

			expect(element).not.toBeInstanceOf(HTMLElement);
		})
	})
});
