/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Component, Input } from '@angular/core';
import { CommandOrder, CommandResource } from '@alfa-client/command-shared';

@Component({
	selector: 'alfa-historie-item-wiedervorlage',
	templateUrl: './historie-item-wiedervorlage.component.html',
	styleUrls: ['./historie-item-wiedervorlage.component.scss'],
})
export class HistorieItemWiedervorlageComponent {

	@Input() command: CommandResource;

	headline: string;
	hasBody: boolean = true;

	get wiedervorlage(): any {
		return this.command.body.item;
	}

	ngOnInit(): void {
		this.headline = HISTORIE_TEXT_BY_WIEDERVORLAGE_ORDER[this.command.order];
		this.hasBody = this.hasItemBody();
	}

	hasItemBody(): boolean {
		return this.command.order == CommandOrder.CREATE_WIEDERVORLAGE || this.command.order == CommandOrder.EDIT_WIEDERVORLAGE;
	}
}

const HISTORIE_TEXT_BY_WIEDERVORLAGE_ORDER: { [order: string]: string } = {
	[CommandOrder.CREATE_WIEDERVORLAGE]: 'eine Wiedervorlage hinzugefügt.',
	[CommandOrder.EDIT_WIEDERVORLAGE]: 'eine Wiedervorlage bearbeitet.',
	[CommandOrder.WIEDERVORLAGE_ERLEDIGEN]: 'eine Wiedervorlage als erledigt markiert.',
	[CommandOrder.WIEDERVORLAGE_WIEDEREROEFFNEN]: 'eine Wiedervorlage als offen markiert.'
}