/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { UntypedFormBuilder } from '@angular/forms';
import { KommentarResource, KommentarService } from '@alfa-client/kommentar-shared';
import { createEmptyStateResource } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { createKommentarResource } from 'libs/kommentar-shared/test/kommentar';
import { of } from 'rxjs';
import { KommentarFormService } from './kommentar.formservice';

describe('KommentarFormService', () => {
	let formService: KommentarFormService;
	let kommentarService: Mock<KommentarService>;

	const formBuilder: UntypedFormBuilder = new UntypedFormBuilder();

	beforeEach(() => {
		kommentarService = mock(KommentarService);

		formService = new KommentarFormService(formBuilder, useFromMock(kommentarService));
	})

	it('should create', () => {
		expect(formService).toBeTruthy();
	});

	describe('submit', () => {

		const kommentarResource: KommentarResource = createKommentarResource();

		beforeEach(() => {
			kommentarService.createKommentar.mockReturnValue(of(createEmptyStateResource()));
			kommentarService.editKommentar.mockReturnValue(of(createEmptyStateResource()));
		})

		it('should call createKommentar', () => {
			formService.source = null;

			formService.submit();

			expect(kommentarService.createKommentar).toHaveBeenCalled();
		})

		it('should call editKommentar', () => {
			formService.source = kommentarResource;

			formService.submit();

			expect(kommentarService.editKommentar).toHaveBeenCalled();
		})
	})
})