/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { Store } from '@ngrx/store';
import { Subject } from 'rxjs';
import { createRouteData } from './../../../test/navigation-test-factory';
import { NavigationFacade } from './navigation.facade';
import { RouteData } from './navigation.models';
import { currentRouteData } from './navigation.selectors';

describe('NavigationFacade', () => {
	let facade: NavigationFacade;
	let store: Mock<Store>;

	let selectionSubject: Subject<any>;

	beforeEach(() => {
		store = mock(Store);

		selectionSubject = new Subject();

		store.select.mockReturnValue(selectionSubject);
		store.dispatch = jest.fn();

		facade = new NavigationFacade(useFromMock(store));
	})

	describe('getCurrentRouteData', () => {

		const routeData: RouteData = createRouteData();

		it('should call selector', (done) => {
			facade.getCurrentRouteData().subscribe(() => {
				expect(store.select).toHaveBeenCalledWith(currentRouteData);
				done();
			});

			selectionSubject.next(routeData);
		})

		it('should return selected value', (done) => {
			facade.getCurrentRouteData().subscribe(routeData => {
				expect(routeData).toBe(routeData);
				done();
			});

			selectionSubject.next(routeData);
		});
	})
})