/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { createRouteData } from './../../../test/navigation-test-factory';
import { RouteData } from './navigation.models';
import { initialState, NavigationState, reducer } from './navigation.reducer';

import * as NavigationActions from './navigation.actions';

describe('Navigation Reducer', () => {

	describe('on get updateRouteData action', () => {

		it('should set route data', () => {
			const routeData: RouteData = createRouteData();
			const action = NavigationActions.updateCurrentRouteData({ routeData });

			const result: NavigationState = reducer(initialState, action);

			expect(result.currentRouteData).toBe(routeData);
		})

		it('should return original state if route data not changed', () => {
			const currentRouteData: RouteData = createRouteData();
			const currentState: NavigationState = { ...initialState, currentRouteData };

			const action = NavigationActions.updateCurrentRouteData({ routeData: createRouteData() });
			const result: NavigationState = reducer(currentState, action);

			expect(result.currentRouteData).toStrictEqual(currentRouteData);
		})
	})
});