/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Component, EventEmitter, Input, Output } from '@angular/core';
import { getPostfachNachrichtenCount, PostfachMailListResource, PostfachNachrichtenCount } from '@alfa-client/postfach-shared';

@Component({
	selector: 'alfa-postfach-mail-pdf-button',
	templateUrl: './postfach-mail-pdf-button.component.html',
	styleUrls: ['./postfach-mail-pdf-button.component.scss'],
})
export class PostfachMailPdfButtonComponent {

	@Input() isDownloadInProgress: boolean;
	@Input() set postfachMailListResource(postfachMailListResource: PostfachMailListResource) {
		this.setPostfachMailCount(postfachMailListResource);
		this.setLabel();
	}
	@Input() showButtonWithLabel: boolean;

	@Output() downloadPdf: EventEmitter<void> = new EventEmitter<void>();

	postfachMailCount: PostfachNachrichtenCount;
	label: string = undefined;

	readonly postfachNachrichtenCount = PostfachNachrichtenCount;

	setPostfachMailCount(postfachMailListResource: PostfachMailListResource): void {
		this.postfachMailCount = getPostfachNachrichtenCount(postfachMailListResource)
	}

	setLabel(): void {
		if (this.postfachMailCount != PostfachNachrichtenCount.NONE) {
			this.label = this.postfachMailCount == PostfachNachrichtenCount.SINGLE ? 'Nachricht exportieren' : 'Nachrichten exportieren';
		}
	}
}