/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { AbstractControl, UntypedFormGroup } from '@angular/forms';
import { isNil } from 'lodash-es';
import { ApiError, Issue } from '../tech.model';
import { replacePlaceholder } from '../tech.util';
import { ValidationMessageCode, VALIDATION_MESSAGES } from './tech.validation.messages';

export function isValidationError(issue: Issue): boolean {
	return issue.messageCode.includes('javax.validation.constraints')
}

export function setValidationError(form: UntypedFormGroup, issue: Issue, pathPrefix?: string) {
	const control = getControlForIssue(form, issue, pathPrefix);

	control.setErrors({ [issue.messageCode]: issue });
	control.markAsTouched();
}

export function getControlForIssue(form: UntypedFormGroup, issue: Issue, pathPrefix?: string): AbstractControl {
	const fieldPath = pathPrefix ? issue.field.substring(pathPrefix.length + 1) : issue.field;

	let curControl: AbstractControl = form;
	fieldPath.split('.').forEach(field => curControl = (<UntypedFormGroup>curControl).controls[field]);

	return curControl;
}

export function getMessageForIssue(label: string, issue: Issue) {
	let msg = VALIDATION_MESSAGES[issue.messageCode];

	if (isNil(msg)) {
		console.warn('No message for code ' + issue.messageCode + ' found.');
		return issue.messageCode;
	}

	msg = replacePlaceholder(msg, 'field', label);
	issue.parameters.forEach(param =>
		msg = replacePlaceholder(msg, param.name, param.value)
	);
	return msg;
}

export function getMessageCode(apiError: ApiError): string {
	return apiError.issues[0].messageCode;
}

export function isValidationFieldFileSizeExceedError(error: any){
	return getMessageCode(error) === ValidationMessageCode.VALIDATION_FIELD_FILE_SIZE_EXCEEDED;
}