/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { DebugElement, Type } from '@angular/core';
import { ComponentFixture } from '@angular/core/testing';
import { By } from '@angular/platform-browser';

export function getElementFromFixtureByType<T>(fixture: ComponentFixture<any>, component: Type<T>): T {
	return getDebugElementFromFixtureByType(fixture, component).componentInstance as T;
}

function getDebugElementFromFixtureByType<T>(fixture: ComponentFixture<any>, component: Type<T>): DebugElement {
	return fixture.debugElement.query(By.directive(component));
}

export function getElementFromFixture(fixture: ComponentFixture<any>, htmlElement: string): any {
	return fixture.nativeElement.querySelector(htmlElement);
}

export function getElementsFromFixture(fixture: ComponentFixture<any>, htmlElement: string): any {
	return fixture.nativeElement.querySelectorAll(htmlElement);
}

export function dispatchEventFromFixture(fixture: ComponentFixture<any>, elementSelector: string, event: string): void {
	const element: DebugElement = getDebugElementFromFixtureByCss(fixture, elementSelector)
	element.nativeElement.dispatchEvent(new Event(event));
}

export function getDebugElementFromFixtureByCss(fixture: ComponentFixture<any>, query: string): DebugElement {
	return fixture.debugElement.query(By.css(query));
}

export function getElementFromDomRoot(fixture: ComponentFixture<any>, htmlElement: string): any {
	return fixture.nativeElement.parentNode.querySelector(htmlElement);
}

function getDebugElementFromFixtureByDirective(fixture: ComponentFixture<any>, query: Type<any>): DebugElement {
	return fixture.debugElement.query(By.directive(query));
}

export function getMockComponent<T>(fixture: ComponentFixture<unknown>, component: Type<T>): T {
	return <T> getDebugElementFromFixtureByDirective(fixture, component).componentInstance;
}