/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { TestBed } from '@angular/core/testing';
import { MatDialog } from '@angular/material/dialog';
import { ApiError } from '@alfa-client/tech-shared';
import { createApiError } from 'libs/tech-shared/test/error';
import { BasicDialogComponent } from '../basic-dialog/basic-dialog.component';
import { DialogService } from '../dialog/dialog.service';
import { FixedDialogComponent } from '../fixed-dialog/fixed-dialog.component';
import { InternalServerErrorDialogComponent } from '../notification/internal-server-error-dialog/internal-server-error-dialog.component';

describe('DialogService', () => {
	let service: DialogService;

	const component = <any>{ name: 'RepraesentantEinerComponent' };
	const dialog = {
		open: jest.fn(),
		getDialogById: jest.fn(),
		closeAll: jest.fn()
	}

	beforeEach(() => {
		TestBed.configureTestingModule({
			providers: [
				{
					provide: MatDialog,
					useValue: dialog
				}
			]
		});
		service = TestBed.inject(DialogService);
	});

	it('should be created', () => {
		expect(service).toBeTruthy();
	});


	describe('open', () => {

		const config = <any>{ id: 'ZumBeispiel' };

		it('should call dialog open with config', () => {
			service.open(component, config);

			expect(dialog.open).toHaveBeenCalledWith(component, config);
		})
	})

	describe('openFixed', () => {

		const data: any = {};

		it('should call dialog open with config', () => {
			service.openFixed(data);

			expect(dialog.open).toHaveBeenCalledWith(FixedDialogComponent, { ...service.FIXED_DIALOG_CONFIG, data });
		})

		it('should close refs if defined', () => {
			const refMock = <any>{ close: jest.fn() };
			service.fixedSingleDialogRef = refMock;

			service.openFixed(data);

			expect(refMock.close).toHaveBeenCalled();
		})
	})

	describe('openBasic', () => {

		const data: any = {};

		it('should call dialog open with config', () => {
			service.openBasic(data);

			expect(dialog.open).toHaveBeenCalledWith(BasicDialogComponent, { ...service.BASIC_DIALOG_CONFIG, data });
		})
	})

	describe('openApiError', () => {

		const apiError: ApiError = createApiError();

		it('should call dialog open with config', () => {
			service.openApiErrorInfo(apiError);

			expect(dialog.open).toHaveBeenCalledWith(InternalServerErrorDialogComponent, { ...service.SERVER_ERROR_DIALOG_CONFIG, data: apiError });
		})
	})

	describe('closeAll', () => {

		it('should close all open dialogs', () => {
			service.closeAll();

			expect(dialog.closeAll).toHaveBeenCalled();
		})
	})
});
