/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatButtonModule } from '@angular/material/button';
import { NoopAnimationsModule } from '@angular/platform-browser/animations';
import { ApiRootLinkRel } from '@alfa-client/api-root-shared';
import { HasLinkPipe, createEmptyStateResource, createStateResource } from '@alfa-client/tech-shared';
import { getElementFromDomRoot, getElementFromFixture } from '@alfa-client/test-utils';
import { UiModule } from '@alfa-client/ui';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { UserSettingsDarkmodeContainerComponent } from './user-settings-darkmode-container/user-settings-darkmode-container.component';
import { UserSettingsEmailBenachrichtigungContainerComponent } from './user-settings-email-benachrichtigung-container/user-settings-email-benachrichtigung-container.component';
import { UserSettingsMenuButtonComponent } from './user-settings-menu-button/user-settings-menu-button.component';
import { UserSettingsComponent } from './user-settings.component';

describe('UserSettingsComponent', () => {
	let component: UserSettingsComponent;
	let fixture: ComponentFixture<UserSettingsComponent>;

	const menuButton: string = getDataTestIdOf('menu-button');
	const emailBenachrichtigung: string = getDataTestIdOf('email-benachrichtigung-toggle');

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			imports: [
				NoopAnimationsModule,
				MatButtonModule,
				UiModule
			],
			declarations: [
				HasLinkPipe,
				UserSettingsComponent,
				MockComponent(UserSettingsMenuButtonComponent),
				MockComponent(UserSettingsEmailBenachrichtigungContainerComponent),
				MockComponent(UserSettingsDarkmodeContainerComponent),
			],
		}).compileComponents();

		fixture = TestBed.createComponent(UserSettingsComponent);
		component = fixture.componentInstance;
		component.apiRoot = createEmptyStateResource()
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('Toggle for "email benachrichtigung"', () => {

		it('should show if link exists', () => {
			component.apiRoot = createStateResource(createApiRootResource([ApiRootLinkRel.CURRENT_USER]));
			fixture.detectChanges();

			getElementFromFixture(fixture, menuButton).click();
			const element = getElementFromDomRoot(fixture, emailBenachrichtigung);

			expect(element).toBeInTheDocument();
		})

		it('should hide if link not exists', () => {
			component.apiRoot = createStateResource(createApiRootResource());

			getElementFromFixture(fixture, menuButton).click();
			const element = getElementFromDomRoot(fixture, emailBenachrichtigung);

			expect(element).not.toBeInTheDocument();
		})
	})
});