/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Component, Input } from '@angular/core';
import { allEmpty, formatFullDate, isParsableToDate } from '@alfa-client/tech-shared';
import { Antragsteller } from '@alfa-client/vorgang-shared';
import { isNull } from 'lodash-es';

@Component({
	selector: 'alfa-vorgang-detail-antragsteller',
	templateUrl: './vorgang-detail-antragsteller.component.html',
	styleUrls: ['./vorgang-detail-antragsteller.component.scss']
})
export class VorgangDetailAntragstellerComponent {

	private readonly TRIM_TO_ONE_SPACE = /\s{2,}/g;

	@Input() antragsteller: Antragsteller;

	get antragstellerExists(): boolean {
		if (isNull(this.antragsteller)) {
			return false;
		}
		if (allEmpty(this.name, this.antragstellerStrasseHausnummer, this.antragstellerPlzOrt, this.geburt, this.antragsteller.telefon ,this.antragsteller.email)) {
			return false;
		}
		return true;
	}

	get name(): string {
		return [this.antragsteller.anrede, this.antragsteller.vorname, this.antragsteller.nachname].join(' ').trim().replace(this.TRIM_TO_ONE_SPACE, ' ');
	}

	get antragstellerStrasseHausnummer(): string {
		return [this.antragsteller.strasse, this.antragsteller.hausnummer].join(' ').trim();
	}

	get antragstellerPlzOrt(): string {
		return [this.antragsteller.plz, this.antragsteller.ort].join(' ').trim();
	}

	get geburt(): string {
		return this.geburtsdatum && this.antragsteller.geburtsort
			? `${this.geburtsdatum} in ${this.antragsteller.geburtsort}`
			: this.geburtsdatum || this.antragsteller.geburtsort;
	}

	get geburtsdatum(): string {
		if (isParsableToDate(this.antragsteller.geburtsdatum)) {
			return formatFullDate(new Date(this.antragsteller.geburtsdatum));
		}
		return this.antragsteller.geburtsdatum;
	}
}