import { UntypedFormBuilder } from '@angular/forms';
import { NavigationService } from '@alfa-client/navigation-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { VorgangListService } from '@alfa-client/vorgang-shared';
import { of } from 'rxjs';
import { VorgangFilterFormService } from './vorgang-filter-menu.formservice';

describe('VorgangFilterFormservice', () => {

	let formService: VorgangFilterFormService;;
	let navigationService: Mock<NavigationService>;
	let vorgangListService;

	beforeEach(() => {
		vorgangListService = { ...mock(VorgangListService), getSearchString: () => of('searchString') };
		navigationService = mock(NavigationService);
		vorgangListService.getSelectedFilterAsRouteParam.mockReturnValue(of('alle'));

		formService = new VorgangFilterFormService(new UntypedFormBuilder(), useFromMock(navigationService), useFromMock(vorgangListService));
	})

	it('should create', () => {
		expect(formService).toBeTruthy();
	})

	describe('init', () => {

		beforeEach(() => {
			formService.subscribeToFilterChanges = jest.fn();
			formService.subscribeToValueChanges = jest.fn();
		})

		it('should create filter form controls', () => {
			expect(formService.form).not.toBeUndefined();
		})

		it('should subscribe to filter changes', () => {
			formService.init();

			expect(formService.subscribeToFilterChanges).toHaveBeenCalled();
		})

		it('should subscribe to value changes', () => {
			formService.init();

			expect(formService.subscribeToValueChanges).toHaveBeenCalled();
		})
	})

	describe('subscribe to filter changes', () => {

		it('should call vorgangListService getSelectedFilterAsRouteParam', () => {
			formService.subscribeToFilterChanges();

			expect(vorgangListService.getSelectedFilterAsRouteParam).toHaveBeenCalled();
		})

		it('should patch form control value', () => {
			formService.subscribeToFilterChanges();

			expect(formService.form.controls['filter'].value).toEqual('alle');
		})
	})

	describe('subscribe to value changes', () => {

		const patchedFilter: string = 'meine';

		it('should call navigationService navigateByFilter', () => {
			formService.subscribeToValueChanges();

			formService.form.controls['filter'].patchValue(patchedFilter);

			expect(navigationService.navigateByFilter).toHaveBeenCalledWith(patchedFilter);
		})
	})
})