/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { TestBed } from '@angular/core/testing';
import { ActivatedRouteSnapshot, CanActivateFn, Router, RouterStateSnapshot, UrlSegment, UrlTree } from '@angular/router';
import { AuthService } from '@alfa-client/app-shared';
import { mock, useFromMock } from '@alfa-client/test-utils';

import * as VorgangNavigationUtil from 'libs/vorgang-shared/src/lib/vorgang-navigation.util';
import * as Guard from './vorgang-list-page.guard';

const next: ActivatedRouteSnapshot = {} as unknown as ActivatedRouteSnapshot;
const state: RouterStateSnapshot = { root: { url: [<UrlSegment>{}] } } as unknown as RouterStateSnapshot;

describe('VorgangListPageGuard', () => {
	const router = mock(Router);
	const authService = mock(AuthService);

	beforeEach(() => {
		TestBed.configureTestingModule({
			providers: [
				{
					provide: Router,
					useValue: router
				},
				{
					provide: AuthService,
					useValue: authService
				}
			]
		});

	});


	it('should be created', () => {
		const guard = TestBed.runInInjectionContext(() => Guard.vorgangListPageGuard(next, state)) as unknown as CanActivateFn;

		expect(guard).toBeTruthy();
	});

	describe('vorgangListPageGuard', () => {

		it('should not call buildDefaultUrlTreeFromLocalStorage', () => {
			const spy = jest.spyOn(Guard, 'buildDefaultUrlTreeFromLocalStorage');
			state.root.url = [<UrlSegment>{ path: 'test' }];

			TestBed.runInInjectionContext(() => Guard.vorgangListPageGuard(next, state)) as unknown as CanActivateFn;

			expect(spy).not.toHaveBeenCalled();
		})

		it('should call buildDefaultUrlTreeFromLocalStorage', () => {
			const spy = jest.spyOn(Guard, 'buildDefaultUrlTreeFromLocalStorage');
			state.root.url = null;

			TestBed.runInInjectionContext(() => Guard.vorgangListPageGuard(next, state)) as unknown as CanActivateFn;

			expect(spy).toHaveBeenCalled();
		})

	})

	describe('buildDefaultUrlTreeFromLocalStorage', () => {
		const buildPathSegmentsFromLocalStorage = jest.spyOn(VorgangNavigationUtil, 'buildPathSegmentsFromLocalStorage');

		it('should call hasNoValidToken', () => {
			const spy = jest.spyOn(authService, 'hasNoValidToken');

			TestBed.runInInjectionContext(() => Guard.buildDefaultUrlTreeFromLocalStorage(useFromMock(router), useFromMock(authService)));

			expect(spy).toHaveBeenCalled();
		})

		it('should call router.createUrlTree', () => {
			jest.spyOn(authService, 'hasNoValidToken').mockReturnValue(false);

			TestBed.runInInjectionContext(() => Guard.buildDefaultUrlTreeFromLocalStorage(useFromMock(router), useFromMock(authService)));

			expect(router.createUrlTree).toHaveBeenCalled();
		})

		it('should call VorgangNavigationUtil.buildPathSegmentsFromLocalStorage', () => {
			jest.spyOn(authService, 'hasNoValidToken').mockReturnValue(false);

			TestBed.runInInjectionContext(() => Guard.buildDefaultUrlTreeFromLocalStorage(useFromMock(router), useFromMock(authService)));

			expect(buildPathSegmentsFromLocalStorage).toHaveBeenCalled();
		})

		it('should return URLTree if access token is valid', () => {
			jest.spyOn(authService, 'hasNoValidToken').mockReturnValue(false);
			const pathSegments: string[] = ['meine', 'neu'];
			buildPathSegmentsFromLocalStorage.mockReturnValue(pathSegments);

			TestBed.runInInjectionContext(() => Guard.buildDefaultUrlTreeFromLocalStorage(useFromMock(router), useFromMock(authService)));

			expect(router.createUrlTree).toHaveBeenCalledWith(pathSegments);
		})

		it('should return true if it has no valid access token', () => {
			jest.spyOn(authService, 'hasNoValidToken').mockReturnValue(true);

			const result: boolean | UrlTree = Guard.buildDefaultUrlTreeFromLocalStorage(useFromMock(router), useFromMock(authService));

			expect(result).toBeTruthy();
		})
	})

});
