import { ComponentFixture, TestBed } from '@angular/core/testing';
import { HintContainerComponent } from '@alfa-client/hint';
import { HasLinkPipe, createStateResource } from '@alfa-client/tech-shared';
import { existsAsHtmlElement, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { VorgangListSearchComponent } from './vorgang-list-search.component';

describe('VorgangListSearchComponent', () => {
	let component: VorgangListSearchComponent;
	let fixture: ComponentFixture<VorgangListSearchComponent>;

	const hintContainer: string = getDataTestIdOf('hint-container');
	const emptySearchResult: string = getDataTestIdOf('vorgang-list-empty-search-result');

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				VorgangListSearchComponent,
				HasLinkPipe,
				MockComponent(HintContainerComponent)
			],
		}).compileComponents();

		fixture = TestBed.createComponent(VorgangListSearchComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('if hints link is not present', () => {

		it('should show search result', () => {
			component.apiRootStateResource = createStateResource(createApiRootResource());

			fixture.detectChanges();

			existsAsHtmlElement(fixture, emptySearchResult);
		})

		it('should hide hint container', () => {
			component.apiRootStateResource = createStateResource(createApiRootResource());

			fixture.detectChanges();

			notExistsAsHtmlElement(fixture, hintContainer);
		})
	})
});
