package de.ozgcloud.alfa.logging;

import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.AfterThrowing;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.springframework.stereotype.Component;

import de.itvsh.kop.common.logging.AspectLoggingUtils;

@Aspect
@Component
public class AlfaLoggingAspect extends AlfaAspectPointcuts {

	@Before("anyPublicServiceMethod() || anyPublicRestControllerMethod()")
	public void onLoggedMethod(JoinPoint joinPoint) {
		AspectLoggingUtils.log(joinPoint);
	}

	@AfterThrowing(pointcut = "anyPublicServiceMethod() || anyPublicRestControllerMethod()", throwing = "ex")
	public void afterExceptionInLoggedMethod(JoinPoint joinPoint, Exception ex) {
		AspectLoggingUtils.logException(joinPoint, ex);
	}

	@AfterReturning(pointcut = "anyPublicServiceMethod() || anyPublicRestControllerMethod()", returning = "returnValue")
	public void afterLoggedMethod(JoinPoint joinPoint, Object returnValue) {
		AspectLoggingUtils.logReturnValue(joinPoint, returnValue);
	}

}
