package de.ozgcloud.alfa.loeschanforderung;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserRole;

class LoeschAnforderungPreAuthorizeHandlerTest {

	@InjectMocks
	private LoeschAnforderungPreAuthorizeHandler handler;

	@Mock
	private LoeschAnforderungService loeschAnforderungService;
	@Mock
	private CurrentUserService currentUserService;

	@Nested
	class TestHasPermissionForOrder {

		private final LoeschAnforderung loeschAnforderung = LoeschAnforderungTestFactory.create();

		@DisplayName("Has permission for VORGANG_LOESCHEN")
		@Nested
		class TestVorgangLoeschen {

			@BeforeEach
			void init() {
				when(loeschAnforderungService.getById(LoeschAnforderungTestFactory.ID)).thenReturn(loeschAnforderung);
			}

			@Test
			void shouldLoadLoeschAnforderung() {
				handler.hasPermissionForOrder(LoeschAnforderungTestFactory.ID, CommandOrder.VORGANG_LOESCHEN);

				verify(loeschAnforderungService).getById(LoeschAnforderungTestFactory.ID);
			}

			@Test
			void shouldCallLoeschAnforderungService() {
				handler.hasPermissionForOrder(LoeschAnforderungTestFactory.ID, CommandOrder.VORGANG_LOESCHEN);

				verify(loeschAnforderungService).isAllowedToVorgangLoeschen(loeschAnforderung);
			}

			@Test
			void shouldReturnFalseIfNotAllowed() {
				when(loeschAnforderungService.isAllowedToVorgangLoeschen(loeschAnforderung)).thenReturn(false);

				var result = handler.hasPermissionForOrder(LoeschAnforderungTestFactory.ID, CommandOrder.VORGANG_LOESCHEN);

				assertThat(result).isFalse();
			}

			@Test
			void shouldReturnTrueIfAllowed() {
				when(loeschAnforderungService.isAllowedToVorgangLoeschen(loeschAnforderung)).thenReturn(true);

				var result = handler.hasPermissionForOrder(LoeschAnforderungTestFactory.ID, CommandOrder.VORGANG_LOESCHEN);

				assertThat(result).isTrue();
			}
		}

		@DisplayName("Has permission for LOESCH_ANFORDERUNG_ZURUECKNEHMEN")
		@Nested
		class TestVorgangLoeschenZuruecknehmen {

			@Test
			void shouldReturnTrue() {
				when(currentUserService.hasRole(UserRole.VERWALTUNG_USER)).thenReturn(true);

				var result = handler.hasPermissionForOrder(LoeschAnforderungTestFactory.ID, CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN);

				assertThat(result).isTrue();
			}

			@Test
			void shouldReturnFalseMissingRole() {
				when(currentUserService.hasRole(UserRole.VERWALTUNG_USER)).thenReturn(false);

				var result = handler.hasPermissionForOrder(LoeschAnforderungTestFactory.ID, CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN);

				assertThat(result).isFalse();
			}

			@Test
			void shouldReturnFalseForNotAllowedCommandOrder() {
				var result = handler.hasPermissionForOrder(LoeschAnforderungTestFactory.ID, CommandOrder.CREATE_ATTACHED_ITEM);

				assertThat(result).isFalse();
			}
		}
	}
}