package de.ozgcloud.alfa.statistic;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

class StatisticControllerTest {

	@InjectMocks
	private StatisticController controller;

	@Mock
	private StatisticService service;

	@Nested
	class TestGetVorgaengeStatistic {

		private static final int ZU_LOESCHEN_COUNT = ByStatusTestFactory.ZU_LOESCHEN_COUNT + 1;

		private final Statistic getVorgangResult = StatisticTestFactory.createBuilder()
				.byStatus(ByStatusTestFactory.createBuilder().zuLoeschen(ZU_LOESCHEN_COUNT).build()).build();
		private final Statistic countVorgangResult = StatisticTestFactory.create();

		@BeforeEach
		void beforeEach() {
			when(service.countVorgang()).thenReturn(countVorgangResult);
			when(service.getVorgaengeStatistic()).thenReturn(getVorgangResult);
		}

		@Test
		void shouldCallGetVorgaengeStatistic() {
			controller.getVorgaengeStatistic();

			verify(service).getVorgaengeStatistic();
		}

		// TODO sebo: später löschen after only new api will be in use
		@Test
		void shouldCallCountVorgang() {
			controller.getVorgaengeStatistic();

			verify(service).countVorgang();
		}

		@Test
		void shouldHaveExistsWiedervorlageOverdue() {
			var statistic = controller.getVorgaengeStatistic();

			assertThat(statistic.isExistsWiedervorlageOverdue()).isEqualTo(StatisticTestFactory.EXISTS_WIEDERVORLAGE_OVERDUE);
		}

		@Test
		void shouldHaveWiedervorlagenCount() {
			var statistic = controller.getVorgaengeStatistic();

			assertThat(statistic.getWiedervorlagen()).isEqualTo(StatisticTestFactory.COUNT_WIEDERVORLAGEN);
		}

		@Test
		void shouldHaveCountByVerworfenStatus() {
			var statistic = controller.getVorgaengeStatistic();

			assertThat(statistic.getByStatus().getVerworfen()).isEqualTo(ByStatusTestFactory.VERWORFEN_COUNT);
		}

		@Test
		void shouldHaveCountByInBearbeitungStatus() {
			var statistic = controller.getVorgaengeStatistic();

			assertThat(statistic.getByStatus().getInBearbeitung()).isEqualTo(ByStatusTestFactory.IN_BEARBEITUNG_COUNT);
		}

		@Test
		void shouldHaveCountByAngenommenStatus() {
			var statistic = controller.getVorgaengeStatistic();

			assertThat(statistic.getByStatus().getAngenommen()).isEqualTo(ByStatusTestFactory.ANGENOMMEN_COUNT);
		}

		@Test
		void shouldHaveCountByBeschiedenStatus() {
			var statistic = controller.getVorgaengeStatistic();

			assertThat(statistic.getByStatus().getBeschieden()).isEqualTo(ByStatusTestFactory.BESCHIEDEN_COUNT);
		}

		@Test
		void shouldHaveCountByAbgeschlossenStatus() {
			var statistic = controller.getVorgaengeStatistic();

			assertThat(statistic.getByStatus().getAbgeschlossen()).isEqualTo(ByStatusTestFactory.ABGESCHLOSSEN_COUNT);
		}

		@Test
		void shouldHaveCountByZuLoeschenStatus() {
			var statistic = controller.getVorgaengeStatistic();

			assertThat(statistic.getByStatus().getZuLoeschen()).isEqualTo(ZU_LOESCHEN_COUNT);
		}

		@Test
		void shouldHaveCountByNeuStatus() {
			var statistic = controller.getVorgaengeStatistic();

			assertThat(statistic.getByStatus().getNeu()).isEqualTo(ByStatusTestFactory.NEU_COUNT);
		}
	}

}