package de.ozgcloud.alfa.statistic;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

class StatisticMapperTest {

	private final StatisticMapper mapper = Mappers.getMapper(StatisticMapper.class);

	@Nested
	class TestToResult {

		private final String RESULT_STRING = "result";
		private final boolean RESULT_BOOL = true;
		private final int RESULT_INT = 42;

		@Test
		void shouldMapName() {
			var statisticResult = mapper.toResult(GrpcVorgangStatisticResultTestFactory.create());

			assertThat(statisticResult.getName()).isEqualTo(GrpcVorgangStatisticResultTestFactory.RESULT_NAME);
		}

		@Test
		void shouldMapStringResult() {
			var grpcResult = GrpcVorgangStatisticResultTestFactory.createBuilder().setResultStringValue(RESULT_STRING).build();

			var statisticResult = mapper.toResult(grpcResult);

			assertThat(statisticResult.getStringValue()).isEqualTo(RESULT_STRING);
		}

		@Test
		void shouldMapBoolResult() {
			var grpcResult = GrpcVorgangStatisticResultTestFactory.createBuilder().setResultBoolValue(RESULT_BOOL).build();

			var statisticResult = mapper.toResult(grpcResult);

			assertThat(statisticResult.getBoolValue()).isEqualTo(RESULT_BOOL);
		}

		@Test
		void shouldMapIntResult() {
			var grpcResult = GrpcVorgangStatisticResultTestFactory.createBuilder().setResultIntValue(RESULT_INT).build();

			var statisticResult = mapper.toResult(grpcResult);

			assertThat(statisticResult.getIntValue()).isEqualTo(RESULT_INT);
		}

	}
}