/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang;

import static de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory.*;
import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.BaseTypesMapper;
import de.ozgcloud.alfa.common.user.UserId;
import de.ozgcloud.alfa.common.user.UserIdMapper;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;

class VorgangWithEingangMapperTest {

	@Spy
	@InjectMocks
	private final VorgangWithEingangMapper mapper = Mappers.getMapper(VorgangWithEingangMapper.class);

	@Spy
	private final UserIdMapper userIdMapper = Mappers.getMapper(UserIdMapper.class);

	@Spy
	private final BaseTypesMapper baseTypesMapper = Mappers.getMapper(BaseTypesMapper.class);

	@Mock
	private EingangMapper eingangMapper;

	@DisplayName("To VorgangWithEingang")
	@Nested
	class TestToVorgangWithEingang {
		@Test
		void shouldMapCreatedAt() {
			var vorgang = callMapper();

			assertThat(vorgang.getCreatedAt()).isEqualTo(CREATED_AT);
		}

		@Test
		void shouldMapNummer() {
			var vorgang = callMapper();

			assertThat(vorgang.getNummer()).isEqualTo(NUMMER);
		}

		@Test
		void shouldMapAktenzeichen() {
			var vorgang = callMapper();

			assertThat(vorgang.getAktenzeichen()).isEqualTo(AKTENZEICHEN);
		}

		@Test
		void shouldMapStatus() {
			var vorgang = callMapper();

			assertThat(vorgang.getStatus()).isEqualTo(VorgangStatus.NEU);
		}

		@Test
		void shouldMapAssignedTo() {
			var vorgang = callMapper();

			assertThat(vorgang.getAssignedTo()).isEqualTo(UserId.from(ASSIGNED_TO));
		}

		@Test
		void shouldMapClientAttributeHasNewPostfachNachricht() {
			var vorgang = callMapper();

			assertThat(vorgang.isHasNewPostfachNachricht()).isTrue();
		}

		private VorgangWithEingang callMapper() {
			return mapper.toVorgangWithEingang(GrpcVorgangWithEingangTestFactory.create());
		}
	}
}
