package de.ozgcloud.alfa.export;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.stream.Stream;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.vorgang.Eingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.xoev.xdomea.AllgemeineMetadatenType;
import de.xoev.xdomea.DokumentType;
import de.xoev.xdomea.IdentifikationObjektType;
import de.xoev.xdomea.MediumCodeType;

@Component
class DokumentTypeCreator {

	static final String ALLGEMEINE_METADATEN_MEDIUM_CODE = "001";

	@Autowired
	private ExportFileService exportFileService;

	@Autowired
	private UUIDConverter uuidConverter;

	@Autowired
	private VersionTypeCreator versionTypeCreator;

	public Stream<DokumentType> create(VorgangWithEingang vorgang) {
		var ozgFiles = Optional.ofNullable(vorgang.getEingang()).map(Eingang::getId).map(exportFileService::getAllPdfs).map(Stream::toList)
				.orElse(Collections.emptyList());
		List<DokumentType> dokumenten = new ArrayList<>(ozgFiles.size());
		for (var i = 0; i < ozgFiles.size(); i++) {
			dokumenten.add(createDokumentType(i + 1, vorgang, ozgFiles.get(i)));
		}
		return dokumenten.stream();
	}

	DokumentType createDokumentType(int dokumentOrdinalNumber, VorgangWithEingang vorgang, OzgFile ozgFile) {
		var dokument = new DokumentType();
		dokument.setIdentifikation(createIdentifikation(dokumentOrdinalNumber, ozgFile.getId().toString()));
		dokument.setAllgemeineMetadaten(createAllgemeineMetadaten(ozgFile));
		dokument.getVersion().add(versionTypeCreator.create(vorgang, ozgFile));
		return dokument;
	}

	IdentifikationObjektType createIdentifikation(int dokumentOrdinalNumber, String fileId) {
		var identifikation = new IdentifikationObjektType();
		identifikation.setID(uuidConverter.fromObjectId(fileId));
		identifikation.setNummerImUebergeordnetenContainer(Long.valueOf(dokumentOrdinalNumber));
		return identifikation;
	}

	AllgemeineMetadatenType createAllgemeineMetadaten(OzgFile ozgFile) {
		var allgemeineMetadaten = new AllgemeineMetadatenType();
		allgemeineMetadaten.setBetreff(ozgFile.getName());
		allgemeineMetadaten.setKennzeichen(StringUtils.EMPTY);
		allgemeineMetadaten.setBemerkung(StringUtils.EMPTY);
		allgemeineMetadaten.setMedium(createMediumCode());
		return allgemeineMetadaten;
	}

	private MediumCodeType createMediumCode() {
		var mediumCode = new MediumCodeType();
		mediumCode.setCode(ALLGEMEINE_METADATEN_MEDIUM_CODE);
		return mediumCode;
	}
}
