package de.ozgcloud.alfa.export;

import java.io.OutputStream;
import java.util.function.Predicate;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileService;
import de.ozgcloud.alfa.common.binaryfile.FileId;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileService;

@Service
class ExportFileService {

	private static final Predicate<OzgFile> IS_PDF_FILE = file -> file.getContentType().equals(MediaType.APPLICATION_PDF_VALUE);

	@Autowired
	private OzgFileService ozgFileService;

	@Autowired
	private BinaryFileService binaryFileService;

	public Stream<OzgFile> getAllPdfs(String eingangId) {
		var representations = ozgFileService.getRepresentationsByEingang(eingangId);
		var attachments = ozgFileService.getAttachmentsByEingang(eingangId);
		return Stream.concat(representations, attachments).filter(IS_PDF_FILE);
	}

	public void writeOzgFile(FileId fileId, OutputStream outputStream) {
		binaryFileService.writeFileContent(fileId, outputStream);
	}

}
