package de.ozgcloud.alfa.export;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;
import de.xoev.xdomea.PrimaerdokumentType;

class FormatTypeCreatorTest {

	@Spy
	@InjectMocks
	private FormatTypeCreator creator;

	@Mock
	private PrimaerdokumentTypeCreator primaerdokumentTypeCreator;

	@Nested
	class TestCreate {

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
		private final OzgFile ozgFile = OzgFileTestFactory.create();

		private final PrimaerdokumentType primaerdokument = PrimaerdokumentTypeTestFactory.create();

		@BeforeEach
		void init() {
			when(primaerdokumentTypeCreator.create(vorgang, ozgFile)).thenReturn(primaerdokument);
		}

		@Test
		void shouldCreatePrimaerdokument() {
			creator.create(vorgang, ozgFile);

			verify(primaerdokumentTypeCreator).create(vorgang, ozgFile);
		}

		@Test
		void shouldHavePrimaerdokumentType() {
			var format = creator.create(vorgang, ozgFile);

			assertThat(format.getPrimaerdokument()).isEqualTo(primaerdokument);
		}

		@Test
		void shouldHaveName() {
			var format = creator.create(vorgang, ozgFile);

			assertThat(format.getName()).usingRecursiveComparison().isEqualTo(DateiformatCode.PDF.createDateiformatCodeType());
		}

		@Test
		void shouldHaveSonstigerName() {
			var format = creator.create(vorgang, ozgFile);

			assertThat(format.getSonstigerName()).isEmpty();
		}

		@Test
		void shouldHaveVersion() {
			var format = creator.create(vorgang, ozgFile);

			assertThat(format.getVersion()).isEmpty();
		}
	}

}