package de.ozgcloud.alfa.export;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.vorgang.EingangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;

class PrimaerdokumentTypeCreatorTest {

	@Spy
	@InjectMocks
	private PrimaerdokumentTypeCreator creator;

	@Mock
	private ExportFilenameGenerator exportFilenameGenerator;

	@Nested
	class TestCreate {

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
		private final OzgFile ozgFile = OzgFileTestFactory.create();

		@BeforeEach
		void init() {
			when(exportFilenameGenerator.generateExportFilename(ozgFile)).thenReturn(ozgFile.getName());
		}

		@Test
		void shouldGenerateExportFilename() {
			creator.create(vorgang, ozgFile);

			verify(exportFilenameGenerator).generateExportFilename(ozgFile);
		}

		@Test
		void shouldHaveDateiname() {
			var primaerdokument = creator.create(vorgang, ozgFile);

			assertThat(primaerdokument.getDateiname()).isEqualTo(ozgFile.getName());
		}

		@Test
		void shouldHaveDateinameOriginal() {
			var primaerdokument = creator.create(vorgang, ozgFile);

			assertThat(primaerdokument.getDateinameOriginal()).isEqualTo(ozgFile.getName());
		}

		@Test
		void shouldHaveErsteller() {
			var primaerdokument = creator.create(vorgang, ozgFile);

			assertThat(primaerdokument.getErsteller()).isEqualTo(EingangHeaderTestFactory.FORM_ENGINE_NAME);
		}
	}
}